"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var gulp_1 = __importDefault(require("gulp"));
var chalk_1 = __importDefault(require("chalk"));
var fancy_log_1 = __importDefault(require("fancy-log"));
var gulp_babel_1 = __importDefault(require("gulp-babel"));
var gulp_uglify_1 = __importDefault(require("gulp-uglify"));
var gulp_htmlmin_1 = __importDefault(require("gulp-htmlmin"));
var gulp_plumber_1 = __importDefault(require("gulp-plumber"));
var gulp_clean_css_1 = __importDefault(require("gulp-clean-css"));
function createConfig(options) {
    var root = options.root;
    var src = path_1.default.join(root, options.src);
    var publish = path_1.default.join(root, options.publish);
    var staticPath = path_1.default.join(publish, options.static);
    var config = {
        css: {
            src: [src + '/**/*.css'],
            dest: staticPath,
        },
        html: {
            src: [src + '/**/*.@(html|htm)'],
            dest: staticPath,
        },
        js: {
            src: [
                src + '/lib/!(es5)/**/*.@(js|ts|jsx|tsx)',
                src + '/lib/*.@(js|ts|jsx|tsx)',
            ],
            dest: staticPath + '/lib',
        },
        es5: {
            src: [src + '/lib/es5/**/*.@(js|ts|jsx|tsx)'],
            dest: staticPath + '/lib/es5',
        },
        copy: {
            src: [
                src + '/**/!(*.@(html|htm|css|js|ts|jsx|tsx))',
                "!".concat(src, "/**/__tests__/**")
            ],
            dest: staticPath,
        },
        publishCopy: {
            src: [
                root + '/*',
                root + "/!(node_modules|buildportal-script)/**/*",
                "!".concat(root, "/@(node_modules|buildportal-script)/**"),
                "!".concat(path_1.default.join(root, options.publish, '**'))
            ],
            dest: publish,
        },
        publishBabel: {
            src: [
                root + "/!(node_modules|buildportal-script)/**/*.@(js|ts|jsx|tsx)",
                root + '/*.@(js|ts|jsx|tsx)',
                "!".concat(path_1.default.join(root, options.publish, '**'))
            ],
            dest: publish,
        },
    };
    for (var key in options.gulp) {
        if (key in config) {
            // if options.gulp[key] === false, remove this task
            if (options.gulp[key] === false) {
                delete config[key];
            }
            else if (options.gulp[key]) {
                config[key].src = config[key].src.concat(options.gulp[key]);
            }
        }
    }
    return config;
}
var removeBabelRuntimePlugin = function (babelConfig) {
    /**
     * Remove @babel/plugin-transform-runtime
     * 因为 gulp 编译的文件可能不会被 bundle，无法使用 require('@babel/runtime')，所以需要把 runtime 代码内联
     */
    babelConfig.plugins = (babelConfig.plugins || []).filter(function (plugin) {
        if (Array.isArray(plugin)) {
            return plugin[0].includes('runtime') ? false : true;
        }
        else if (typeof plugin === 'string') {
            return plugin.includes('runtime') ? false : true;
        }
        return true;
    });
    return babelConfig;
};
function createGulpTask(options) {
    var config = Object.assign(createConfig(options));
    var minifyCSS = function () {
        if (!config.css) {
            return;
        }
        return gulp_1.default
            .src(config.css.src)
            .pipe((0, gulp_plumber_1.default)())
            .pipe((0, gulp_clean_css_1.default)({}, function (details) {
            var percent = ((details.stats.minifiedSize / details.stats.originalSize) *
                100).toFixed(2);
            var message = "".concat(details.name, "(").concat(chalk_1.default.green(percent), "%)");
            (0, fancy_log_1.default)('gulp-clean-css:', message);
        }))
            .pipe(gulp_1.default.dest(config.css.dest));
    };
    var minifyHTML = function () {
        if (!config.html) {
            return;
        }
        return gulp_1.default
            .src(config.html.src)
            .pipe((0, gulp_plumber_1.default)())
            .pipe((0, gulp_htmlmin_1.default)({
            collapseWhitespace: true,
        }))
            .pipe(gulp_1.default.dest(config.html.dest));
    };
    var minifyES6 = function () {
        if (!config.js) {
            return;
        }
        return gulp_1.default
            .src(config.js.src)
            .pipe((0, gulp_plumber_1.default)())
            .pipe((0, gulp_babel_1.default)(removeBabelRuntimePlugin(options.babel(options))))
            .pipe((0, gulp_uglify_1.default)())
            .pipe(gulp_1.default.dest(config.js.dest));
    };
    var minifyES5 = function () {
        if (!config.es5) {
            return;
        }
        return gulp_1.default
            .src(config.es5.src)
            .pipe((0, gulp_plumber_1.default)())
            .pipe((0, gulp_uglify_1.default)())
            .pipe(gulp_1.default.dest(config.es5.dest));
    };
    var publishCopy = function () {
        if (!config.publishCopy) {
            return;
        }
        return gulp_1.default
            .src(config.publishCopy.src)
            .pipe((0, gulp_plumber_1.default)())
            .pipe(gulp_1.default.dest(config.publishCopy.dest));
    };
    var publishBabel = function () {
        if (!config.publishBabel) {
            return;
        }
        return gulp_1.default
            .src(config.publishBabel.src)
            .pipe((0, gulp_plumber_1.default)())
            .pipe((0, gulp_babel_1.default)(options.babel(options)))
            .pipe(gulp_1.default.dest(config.publishBabel.dest));
    };
    var copy = function () {
        if (!config.copy) {
            return;
        }
        return gulp_1.default
            .src(config.copy.src)
            .pipe((0, gulp_plumber_1.default)())
            .pipe(gulp_1.default.dest(config.copy.dest));
    };
    var parallelList = [
        config.html && minifyHTML,
        config.css && minifyCSS,
        config.es5 && minifyES5,
        config.js && minifyES6,
    ].filter(Boolean);
    var seriesList = [
        config.publishCopy && publishCopy,
        config.publishBabel && publishBabel,
        config.copy && copy,
    ].filter(Boolean);
    return gulp_1.default.series.apply(gulp_1.default, __spreadArray(__spreadArray([], seriesList, false), [gulp_1.default.parallel.apply(gulp_1.default, parallelList)], false));
}
exports.default = createGulpTask;
