"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importDefault(require("react"));
var context_1 = __importDefault(require("../context"));
var ControllerProxy_1 = __importDefault(require("./ControllerProxy"));
var ViewManager = /** @class */ (function (_super) {
    __extends(ViewManager, _super);
    function ViewManager(props, context) {
        var _this = _super.call(this, props, context) || this;
        _this.views = {};
        _this.scrollMap = {};
        _this.addItemIfNeed(props.controller.location.raw);
        return _this;
    }
    ViewManager.prototype.addItemIfNeed = function (key) {
        if (!this.views.hasOwnProperty(key)) {
            this.views[key] = null;
        }
    };
    ViewManager.prototype.clearItemIfNeed = function () {
        var _a = this, views = _a.views, scrollMap = _a.scrollMap;
        var controller = this.props.controller;
        var cache = controller.getAllCache
            ? controller.getAllCache()
            : {};
        for (var key in views) {
            if (!cache.hasOwnProperty(key)) {
                delete views[key];
            }
        }
        for (var key in scrollMap) {
            if (!cache.hasOwnProperty(key)) {
                delete scrollMap[key];
            }
        }
    };
    ViewManager.prototype.UNSAFE_componentWillReceiveProps = function (nextProps) {
        var currentPath = this.props.controller.location.raw;
        var nextPath = nextProps.controller.location.raw;
        if (currentPath !== nextPath) {
            this.scrollMap[currentPath] = window.scrollY;
        }
        this.addItemIfNeed(nextPath);
        this.clearItemIfNeed();
    };
    ViewManager.prototype.renderView = function (path) {
        var controller = this.props.controller;
        var View = controller.View, store = controller.store, location = controller.location, history = controller.history, context = controller.context, matcher = controller.matcher, loader = controller.loader, prefetch = controller.prefetch, handleInputChange = controller.handleInputChange, meta = controller.meta;
        var currentPath = controller.location.raw;
        if (currentPath !== path) {
            return this.views[path];
        }
        var state = store.getState();
        var actions = store.actions;
        var globalContext = {
            ctrl: controller,
            location: location,
            history: history,
            state: state,
            actions: actions,
            preload: context.preload,
            handleInputChange: handleInputChange,
            matcher: matcher,
            loader: loader,
            prefetch: prefetch,
        };
        var view = (react_1.default.createElement(context_1.default.Provider, { value: globalContext },
            react_1.default.createElement(View, { key: "".concat(meta.id, "_").concat(currentPath), state: state, ctrl: controller })));
        if (controller.KeepAlive) {
            this.views[path] = view;
        }
        else if (this.views.hasOwnProperty(path)) {
            delete this.views[path];
        }
        return view;
    };
    ViewManager.prototype.render = function () {
        var _this = this;
        var controller = this.props.controller;
        return (react_1.default.createElement(react_1.default.Fragment, null,
            Object.keys(this.views).map(function (path) {
                return (react_1.default.createElement(ViewItem, { key: path, path: path, isActive: path === controller.location.raw, view: _this.renderView(path), scrollY: _this.scrollMap[path], resetScrollOnMount: controller.resetScrollOnMount }));
            }),
            react_1.default.createElement(ControllerProxy_1.default, { key: controller.meta.id + controller.location.raw, controller: controller })));
    };
    ViewManager.ignoreErrors = true;
    return ViewManager;
}(react_1.default.Component));
exports.default = ViewManager;
var ViewItem = /** @class */ (function (_super) {
    __extends(ViewItem, _super);
    function ViewItem() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.container = null;
        _this.getContainer = function (container) {
            _this.container = container;
        };
        _this.getResetScrollOnMount = function () {
            var resetScrollOnMount = _this.props.resetScrollOnMount;
            return resetScrollOnMount === void 0 ? true : !!resetScrollOnMount;
        };
        return _this;
    }
    ViewItem.prototype.shouldComponentUpdate = function (nextProps) {
        if (!nextProps.isActive) {
            ;
            this.container.style.display = 'none';
        }
        else {
            if (!this.props.isActive) {
                ;
                this.container.style.display = '';
                window.scroll(0, this.props.scrollY);
            }
        }
        return nextProps.isActive;
    };
    ViewItem.prototype.componentDidMount = function () {
        if (this.getResetScrollOnMount()) {
            window.scroll(0, 0);
        }
    };
    ViewItem.prototype.render = function () {
        return (react_1.default.createElement("div", { className: "imvc-view-item", ref: this.getContainer }, this.props.view));
    };
    ViewItem.ignoreErrors = true;
    return ViewItem;
}(react_1.default.Component));
