"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBabelConfig = exports.defaultConfig = void 0;
var fs_1 = __importDefault(require("fs"));
var vm_1 = __importDefault(require("vm"));
var path_1 = __importDefault(require("path"));
var babel_1 = __importDefault(require("./babel"));
var config_defaults_1 = __importDefault(require("./config.defaults"));
var util_1 = require("../util");
var core_1 = require("@babel/core");
var config_defaults_2 = require("./config.defaults");
Object.defineProperty(exports, "defaultConfig", { enumerable: true, get: function () { return __importDefault(config_defaults_2).default; } });
exports.getBabelConfig = babel_1.default;
var Extension;
(function (Extension) {
    Extension["JS"] = "js";
    Extension["TS"] = "ts";
    Extension["JSX"] = "jsx";
    Extension["TSX"] = "tsx";
    Extension["JSON"] = "json";
    Extension["INVALID"] = "invalid";
})(Extension || (Extension = {}));
var preOptions = {};
var configCache = config_defaults_1.default;
function getConfig(options, shouldCompile) {
    if (options === void 0) { options = {}; }
    if (shouldCompile === void 0) { shouldCompile = process.env.NODE_ENV !== 'production'; }
    var canUseCache = (0, util_1.compareObject)(preOptions, options);
    if (canUseCache) {
        return configCache;
    }
    preOptions = options;
    return (configCache = constructConfig(options, shouldCompile));
}
exports.default = getConfig;
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : { default: obj };
}
function constructConfig(options, shouldCompile) {
    var config = null;
    switch (typeof options.config) {
        case 'object':
            config = options.config;
            break;
        case 'string':
            config = shouldCompile
                ? _interopRequireDefault(requireConfig(path_1.default.resolve(options.config)))
                    .default
                : _interopRequireDefault(require(path_1.default.resolve(options.config))).default;
            break;
    }
    return Object.assign({}, config_defaults_1.default, config);
}
function requireConfig(filePath) {
    var clearFilePath = (0, util_1.getClearFilePath)(filePath);
    var _a = getFileInfo(clearFilePath), finalFilePath = _a[0], ets = _a[1];
    if (ets === Extension.INVALID) {
        throw new Error("The config path: ".concat(filePath, " is incorrect"));
    }
    if (ets === Extension.JSON) {
        return require(finalFilePath);
    }
    var babelConfig = __assign(__assign({}, (0, babel_1.default)(config_defaults_1.default)), { filenameRelative: finalFilePath });
    var result = (0, core_1.transformFileSync)(finalFilePath, babelConfig);
    if (result && result.code) {
        try {
            return runCode(result.code, createContext(finalFilePath));
        }
        catch (e) {
            throw e;
        }
    }
    else {
        throw new Error("The file: ".concat(filePath, " has syntax error"));
    }
}
function createContext(filepath) {
    var dir = path_1.default.dirname(filepath);
    var module = { exports: {} };
    var virtualRequire = function (filePath) {
        if (!(0, util_1.isAbsolutePath)(filePath)) {
            return requireConfig(path_1.default.resolve(dir, filePath));
        }
        else {
            return require(filePath);
        }
    };
    // assign require methods to virtual-require
    Object.assign(virtualRequire, require);
    return vm_1.default.createContext(__assign(__assign({}, global), { global: global, console: console, process: process, __filename: filepath, __dirname: dir, exports: module.exports, require: virtualRequire, module: module }));
}
function getFileInfo(filePath) {
    var clearFilePath = (0, util_1.getClearFilePath)(filePath, (0, util_1.getKeys)(Extension).map(function (key) { return Extension[key]; }));
    var finalFilePath = filePath;
    var extension = Extension.INVALID;
    (0, util_1.getKeys)(Extension).some(function (etsKey) {
        var ets = Extension[etsKey];
        if (fs_1.default.existsSync("".concat(clearFilePath, ".").concat(ets))) {
            finalFilePath = "".concat(clearFilePath, ".").concat(ets);
            extension = ets;
            return true;
        }
        return false;
    });
    return [finalFilePath, extension];
}
function runCode(sourceCode, context) {
    sourceCode = sourceCode.replace('"use strict";', '');
    return Function("\n    \"use strict\";\n    return(function({\n      process,\n      __filename,\n      __dirname,\n      exports,\n      require,\n      module\n    }){\n      ".concat(sourceCode, "\n      return module.exports\n    })\n  "))()(context);
}
