import 'whatwg-fetch';
import React from 'react';
import Cookie from 'js-cookie';
import * as shareActions from './actions';
import type { Store, Data, Actions } from 'relite';
import type { HistoryWithBFOL, ILWithBQ, BLWithBQ } from 'create-history';
import type { Controller as AppController, HistoryLocation, Matcher, Loader, CacheStorage } from 'create-app/server';
import type { BaseViewFC, BaseViewClass, Preload, API, SSR, Context, BaseState, Meta, Location, FetchOptions } from '..';
export type EarlyHint = {
    uri: string;
    [key: string]: string;
};
export type EarlyHints = EarlyHint[];
export type BaseActions = typeof shareActions;
/**
 * 绑定 Store 到 View
 * 提供 Controller 的生命周期钩子
 * 提供 fetch 方法
 */
export default class Controller<S extends object, AS extends Actions<BaseState & S>> implements AppController {
    location: Location;
    history: HistoryWithBFOL<BLWithBQ, ILWithBQ>;
    context: Context;
    View: React.ComponentType<any>;
    Model?: any;
    initialState?: any;
    actions?: any;
    store: Store<S & BaseState, BaseActions & AS>;
    SSR?: SSR;
    preload: Preload;
    KeepAlive?: boolean;
    KeepAliveOnPush?: boolean | undefined;
    Loading: BaseViewFC | BaseViewClass;
    API?: API;
    restapi?: string;
    resetScrollOnMount?: boolean;
    meta: Meta;
    proxyHandler?: {
        attach(): void;
        detach(): void;
    };
    deepCloneInitialState: boolean;
    matcher: Matcher;
    loader: Loader;
    getInitialState(state: S & BaseState): any;
    getFinalActions(actions: AS): any;
    shouldComponentCreate?(): void | boolean | Promise<void | boolean>;
    componentWillCreate?(): void | Promise<void>;
    viewWillHydrate?(): void | Promise<void>;
    componentDidFirstMount?(): void | Promise<void>;
    componentDidMount?(): void | Promise<void>;
    pageWillLeave?(location: ILWithBQ): void;
    componentWillUnmount?(): void | Promise<void>;
    pageDidBack?(location: HistoryLocation, context?: Context): void;
    windowWillUnload?(location: ILWithBQ): void;
    errorDidCatch?(error: Error, str: string): void;
    getComponentFallback?(displayName: string, InputComponent: React.ComponentType): React.ReactElement | string | undefined | null;
    getViewFallback?(view?: string): React.ReactElement | string | undefined | null;
    stateDidReuse?(state: S & BaseState): void;
    stateDidChange?(data?: Data<S & BaseState, AS & BaseActions>): void;
    saveToCache(): void;
    removeFromCache(): void;
    getAllCache?(): CacheStorage<this>;
    refreshView?(view?: any): void;
    getContainer?(): HTMLElement | null;
    clearContainer?(): void;
    handleInputChange(path: string, value: S, oldValue: S): S;
    constructor(location: Location, context: Context);
    static get __SYMBOL(): string;
    /**
     * 封装 fetch, https://github.github.io/fetch
     * options.json === false 不自动转换为 json
     * options.timeout:number 超时时间
     * options.timeoutErrorFormatter 超时时错误信息展示格式
     * options.raw 不补全 restfulBasename
     */
    fetch(url: string, options?: FetchOptions): Promise<any>;
    /**
     *
     * 封装 get 请求，方便使用
     */
    get(url: string, params?: Record<string, string | number | boolean>, options?: FetchOptions): Promise<any>;
    /**
     *
     * 封装 post 请求，方便使用
     */
    post(url: string, data?: any, options?: FetchOptions): Promise<any>;
    /**
     * 基于 webpack 构建的 assets.json 获取客户端的静态资源路径
     */
    getClientAssetPath(assetPath: string): string;
    getClientAssetFullPath(assetPath: string): string;
    /**
     * 用于替换 css 中的 publicPath 的占位符
     *
     * 默认为：@public_path
     */
    publicPathPlaceholder: string;
    /**
     * 是否在 preload 里禁用 publicPath
     * 默认为 false，只对 CRS 生效
     * 如果为 true，会直接使用 node.js 服务端的静态资源路径
     */
    disablePublicPathForPreload: boolean;
    /**
     * 预加载 css 样式等资源
     */
    fetchPreload(preload?: Preload): Promise<(void | undefined)[]> | undefined;
    /**
     * 预加载页面的 js bundle
     */
    prefetch(url: string): import("create-app/lib").ControllerConstructor<AppController> | Promise<import("create-app/lib").ControllerConstructor<AppController>> | null;
    prependBasename(pathname: string): string;
    prependPublicPath(pathname: string): string;
    prependRestapi(url: string): string;
    /**
     * 封装重定向方法，根据 server/client 环境不同而选择不同的方式
     * isRaw 是否不拼接 Url，直接使用
     */
    redirect(redirectUrl: string, isRaw?: boolean): void;
    reload(): void;
    cookie(key: string, value?: string, options?: Cookie.CookieAttributes): any;
    getCookie(key: string): any;
    setCookie(key: string, value: string, options?: Cookie.CookieAttributes): void;
    removeCookie(key: string, options?: Cookie.CookieAttributes): void;
    disableEarlyHints: boolean;
    earlyHintsLinks: EarlyHints;
    addEarlyHintsLinks(earlyHints: EarlyHints): void;
    flushHeaders(headers?: Record<string, string>): void;
    init(): Promise<string | JSX.Element | null>;
    destroy(): void;
    initialize(): Promise<JSX.Element | null>;
    bindStoreWithView(): void;
    restore(location: Location, context?: Context): React.ReactElement;
    renderView(View?: React.ComponentType<any>): void;
    render(): React.ReactElement;
}
