"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var createCache_1 = __importDefault(require("./createCache"));
var defaults = {
    timeout: 5 * 60 * 1000,
    max: 5,
    headers: {
        // 缓存的 response headers 默认配置
        'Content-Type': 'text/html',
    },
    key: function (url, req) { return url; },
    debug: false,
};
var callNext = function (req, res, next) { return next(); };
function cacheView(settings) {
    // 只在生产环境，或者开启了 debug = true 的情况下，做缓存
    if (process.env.NODE_ENV !== 'production' && !(settings && settings.debug)) {
        return callNext;
    }
    settings = Object.assign({}, defaults, settings);
    var cache = (0, createCache_1.default)();
    return function (req, res, next) {
        var cacheKey = settings.key(req.originalUrl, req);
        var cacheContent;
        if (cache.get) {
            cacheContent = cache.get(cacheKey);
        }
        // 命中缓存，直接返回结果
        if (cacheContent) {
            res.set(settings.headers);
            res.send(cacheContent);
            return;
        }
        // 劫持 res.render，缓存其结果
        res.sendResponse = res.send;
        res.send = function (body) {
            res.sendResponse(body);
            if (cache)
                cache.put(cacheKey, body, settings.timeout);
            if (settings.max && cache.size() > settings.max) {
                // 如果缓存数大于最大值，删除第一个缓存
                cache.del(cache.keys()[0]);
            }
            return res;
        };
        // 调用下一个中间件
        next();
    };
}
exports.default = cacheView;
