"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = (function () {
    var result;
    var cache = Object.create(null);
    var debug = false;
    var hitCount = 0;
    var missCount = 0;
    var size = 0;
    function _del(key) {
        size--;
        delete cache[key];
    }
    result = {
        put: function (key, value, time, timeoutCallback) {
            if (debug) {
                console.log('caching: %s = %j (@%s)', key, value, time);
            }
            if (typeof time !== 'undefined' &&
                (typeof time !== 'number' || isNaN(time) || time <= 0)) {
                throw new Error('Cache timeout must be a positive number');
            }
            else if (typeof timeoutCallback !== 'undefined' &&
                typeof timeoutCallback !== 'function') {
                throw new Error('Cache timeout callback must be a function');
            }
            var oldRecord = cache[key];
            if (oldRecord) {
                clearTimeout(oldRecord.timeout);
            }
            else {
                size++;
            }
            var record = {
                value: value,
                expire: (time || 0) + Date.now(),
            };
            if (!isNaN(record.expire)) {
                record.timeout = setTimeout(function () {
                    _del(key);
                    if (timeoutCallback) {
                        timeoutCallback(key, value);
                    }
                }, time);
            }
            cache[key] = record;
            return value;
        },
        del: function (key) {
            var canDelete = true;
            var oldRecord = cache[key];
            if (oldRecord) {
                clearTimeout(oldRecord.timeout);
                if (!isNaN(oldRecord.expire) && oldRecord.expire < Date.now()) {
                    canDelete = false;
                }
            }
            else {
                canDelete = false;
            }
            if (canDelete) {
                _del(key);
            }
            return canDelete;
        },
        clear: function () {
            for (var key in cache) {
                clearTimeout(cache[key].timeout);
            }
            size = 0;
            cache = Object.create(null);
            if (debug) {
                hitCount = 0;
                missCount = 0;
            }
        },
        get: function (key) {
            var data = cache[key];
            if (typeof data !== 'undefined') {
                if (isNaN(data.expire) || data.expire >= Date.now()) {
                    if (debug)
                        hitCount++;
                    return data.value;
                }
                else {
                    // free some space
                    if (debug)
                        missCount++;
                    size--;
                    delete cache[key];
                }
            }
            else if (debug) {
                missCount++;
            }
            return null;
        },
        size: function () {
            return size;
        },
        memsize: function () {
            var size = 0;
            for (var _ in cache) {
                size++;
            }
            return size;
        },
        debug: function (bool) {
            debug = bool;
        },
        hits: function () {
            return hitCount;
        },
        misses: function () {
            return missCount;
        },
        keys: function () {
            return Object.keys(cache);
        },
    };
    return result;
});
