"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isIMVCController = exports.compareObject = exports.getClearFilePath = exports.isAbsolutePath = exports.stringToUnit8Array = exports.ab2str = exports.getKeys = exports.getValueByPath = exports.getValue = exports.setValueByPath = exports.setValue = exports.getPath = exports.isThenable = exports.mapValues = exports.isAbsoluteUrl = exports.timeoutReject = exports.toText = exports.toJSON = exports.getFlatList = void 0;
function getFlatList(list) {
    var result = [];
    for (var i = 0; i < list.length; i++) {
        var item = list[i];
        if (Array.isArray(item)) {
            result = result.concat(getFlatList(item));
        }
        else {
            result.push(item);
        }
    }
    return result;
}
exports.getFlatList = getFlatList;
function toJSON(response) {
    // 如果 response 状态异常，抛出错误
    if (!response.ok || response.status !== 200) {
        return Promise.reject(new Error(response.statusText));
    }
    return response.json();
}
exports.toJSON = toJSON;
function toText(response) {
    // 如果 response 状态异常，抛出错误
    if (!response.ok || response.status !== 200) {
        return Promise.reject(new Error(response.statusText));
    }
    return response.text();
}
exports.toText = toText;
function timeoutReject(promise, time, errorMsg) {
    if (time === void 0) { time = 0; }
    var timeoutReject = new Promise(function (_, reject) {
        var err = new Error(errorMsg || "Timeout Error:".concat(time, "ms"));
        setTimeout(function () { return reject(err); }, time);
    });
    return Promise.race([promise, timeoutReject]);
}
exports.timeoutReject = timeoutReject;
function isAbsoluteUrl(url) {
    return url.indexOf('http') === 0 || url.indexOf('//') === 0;
}
exports.isAbsoluteUrl = isAbsoluteUrl;
function mapValues(obj, fn) {
    return Object.keys(obj).reduce(function (result, key) {
        result[key] = fn(obj[key], key);
        return result;
    }, {});
}
exports.mapValues = mapValues;
function isThenable(obj) {
    return obj != null && typeof obj.then === 'function';
}
exports.isThenable = isThenable;
var PATH_SEPARATOR_REGEXP = /\.|\/|:/;
function getPath(path) {
    if (Array.isArray(path))
        return path;
    return path.split(PATH_SEPARATOR_REGEXP);
}
exports.getPath = getPath;
function setValue(obj, _a, value) {
    var key = _a[0], rest = _a.slice(1);
    obj = Array.isArray(obj) ? obj.concat() : Object.assign({}, obj);
    // @ts-ignore
    obj[key] = rest.length > 0 ? setValue(obj[key], rest, value) : value;
    return obj;
}
exports.setValue = setValue;
function setValueByPath(obj, path, value) {
    return setValue(obj, getPath(path), value);
}
exports.setValueByPath = setValueByPath;
function getValue(ret, key) {
    return ret[key];
}
exports.getValue = getValue;
function getValueByPath(obj, path) {
    return getPath(path).reduce(getValue, obj);
}
exports.getValueByPath = getValueByPath;
function getKeys(o) {
    return Object.keys(o);
}
exports.getKeys = getKeys;
function ab2str(buf) {
    return String.fromCharCode.apply(null, Array.from(new Uint16Array(buf)));
}
exports.ab2str = ab2str;
function stringToUnit8Array(str) {
    var arr = [];
    for (var i = 0, j = str.length; i < j; ++i) {
        arr.push(str.charCodeAt(i));
    }
    var tmpUint8Array = new Uint8Array(arr);
    return tmpUint8Array;
}
exports.stringToUnit8Array = stringToUnit8Array;
function isAbsolutePath(path) {
    return path[0] !== '.';
}
exports.isAbsolutePath = isAbsolutePath;
function getClearFilePath(filepath, extensions) {
    if (extensions === void 0) { extensions = ['js']; }
    function replacer(match, p1, p2, offset, str) {
        if (extensions.includes(p2)) {
            return p1;
        }
        else {
            return str;
        }
    }
    return filepath.replace(/^(.*)\.([a-zA-Z]{1,5})$/, replacer);
}
exports.getClearFilePath = getClearFilePath;
function compareObject(a, b) {
    return JSON.stringify(a) === JSON.stringify(b);
}
exports.compareObject = compareObject;
function isIMVCController(fn) {
    return fn.__SYMBOL === 'REACT_IMVC_CONTROLLER';
}
exports.isIMVCController = isIMVCController;
